//
//  TSDailySummaryViewController.m
//  TravelSection
//
//  Created by Michael Herring on 12/17/11.
//  Copyright (c) 2011 Sun, Sea and Sky Factory. All rights reserved.
//

#import "TSDailySummaryViewController.h"
#import "TSReporter.h"

@implementation TSDailySummaryViewController

@synthesize characterView = _characterView;
@synthesize result = _result;
@synthesize intelligenceBar = _intelligenceBar;
@synthesize creativityBar = _creativityBar;
@synthesize technologyBar = _technologyBar;
@synthesize cultureBar = _cultureBar;
@synthesize experienceBar = _experienceBar;
@synthesize stressBar = _stressBar;
@synthesize intelligenceDeltaLabel = _intelligenceDeltaLabel;
@synthesize creativityDeltaLabel = _creativityDeltaLabel;
@synthesize technologyDeltaLabel = _technologyDeltaLabel;
@synthesize cultureDeltaLabel = _cultureDeltaLabel;
@synthesize experienceDeltaLabel = _experienceDeltaLabel;
@synthesize stressDeltaLabel = _stressDeltaLabel;
@synthesize moneyDeltaLabel = _moneyDeltaLabel;
@synthesize sleepButton = _sleepButton;

-(void)initializeAppearance {
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSReporter * reporter = appDelegate.currentReporter;
    //make that character pixelated, yo.
    [[self.characterView layer] setMagnificationFilter:kCAFilterNearest];
    //animate it.
    [self.characterView setAnimationDuration:0.8f];
    [self.characterView setImage:reporter.stationaryImage];
    //set the bars correctly.
    [[self intelligenceBar] setProgress:reporter.intelligence/10.0f animated:NO];
    [[self creativityBar] setProgress:reporter.creativity/10.0f animated:NO];
    [[self technologyBar] setProgress:reporter.technology/10.0f animated:NO];
    [[self cultureBar] setProgress:reporter.culture/100.0f animated:NO];
    [[self experienceBar] setProgress:reporter.experience/100.0f animated:NO];
    [[self stressBar] setProgress:reporter.stress/100.0f animated:NO];
}

-(void)processDailyEvent {
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSReporter * reporter = appDelegate.currentReporter;
    NSDictionary * activity = reporter.currentActivity;
    CGFloat intelligenceDelta = 0.0f, creativityDelta = 0.0f, technologyDelta = 0.0f, cultureDelta = 0.0f, experienceDelta = 0.0f, stressDelta = 0.0f;

    CGFloat moneyDelta = [[activity valueForKey:@"MoneyDelta"] floatValue];
    if ([activity valueForKey:@"TravelMethod"] == nil) {
        if (moneyDelta > 0.0f) {
            CGFloat weightedIntelligenceModifier = reporter.intelligence/10 * [[activity valueForKey:@"IntelligenceMultiplier"] floatValue];
            moneyDelta *= 1.0f + weightedIntelligenceModifier;
            CGFloat weightedCreativityModifier = reporter.creativity/10 * [[activity valueForKey:@"CreativityMultiplier"] floatValue];
            moneyDelta *= 1.0f + weightedCreativityModifier;
            CGFloat weightedTechnologyModifier = reporter.technology/10 * [[activity valueForKey:@"TechnologyMultiplier"] floatValue];
            moneyDelta *= 1.0f + weightedTechnologyModifier;
        }
        //now do the stat differentials
        intelligenceDelta = [[activity valueForKey:@"IntelligenceBonus"] floatValue];
        creativityDelta = [[activity valueForKey:@"CreativityBonus"] floatValue];
        technologyDelta = [[activity valueForKey:@"TechnologyBonus"] floatValue];
        cultureDelta = [[activity valueForKey:@"CultureBonus"] floatValue];
        experienceDelta = [[activity valueForKey:@"ExperienceBonus"] floatValue];
        stressDelta = [[activity valueForKey:@"StressBonus"] floatValue];
        
        //add game bonuses
        if ([appDelegate.activityBonusName isEqualToString:@"Intelligence"]) {
            intelligenceDelta += appDelegate.activityBonusAmount;
        }
        if ([appDelegate.activityBonusName isEqualToString:@"Creativity"]) {
            creativityDelta += appDelegate.activityBonusAmount;
        }
        if ([appDelegate.activityBonusName isEqualToString:@"Technology"]) {
            technologyDelta += appDelegate.activityBonusAmount;
        }
        if ([appDelegate.activityBonusName isEqualToString:@"Culture"]) {
            cultureDelta += appDelegate.activityBonusAmount;
        }
        if ([appDelegate.activityBonusName isEqualToString:@"Experience"]) {
            experienceDelta += appDelegate.activityBonusAmount;
        }
        if ([appDelegate.activityBonusName isEqualToString:@"Stress"]) {
            stressDelta += appDelegate.activityBonusAmount;
        }
        
        //cap things out
        if (reporter.intelligence > 10) intelligenceDelta = 0.0f;
        if (reporter.creativity > 10) creativityDelta = 0.0f;
        if (reporter.technology > 10) technologyDelta = 0.0f;
        if (reporter.culture > 100) cultureDelta = 0.0f;
        if (reporter.experience > 100) experienceDelta = 0.0f;
        if (reporter.stress > 100)stressDelta = 0.0f;
        
    }
    else {
        //travel
        [reporter setCurrentCity:[activity valueForKey:@"Destination"]];
    }
    [reporter setMoney:reporter.money+moneyDelta];
    [reporter setIntelligence:reporter.intelligence+intelligenceDelta];
    [reporter setCreativity:reporter.creativity+creativityDelta];
    [reporter setTechnology:reporter.technology+technologyDelta];
    [reporter setCulture:reporter.culture+cultureDelta];
    [reporter setExperience:reporter.experience+experienceDelta];
    [reporter setStress:reporter.stress+stressDelta];
    NSString * moneyDeltaName = (moneyDelta > 0)?@"earned":@"spent";
    [self.moneyDeltaLabel setText:[NSString stringWithFormat:@"%@ %@\n$%.2f today.",reporter.characterName,moneyDeltaName,fabs(moneyDelta)]];
    //pack up stat boosts and send to anim function
    NSDictionary * statBoostDict = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:[NSNumber numberWithFloat:intelligenceDelta], [NSNumber numberWithFloat:creativityDelta], [NSNumber numberWithFloat:technologyDelta], [NSNumber numberWithFloat:cultureDelta], [NSNumber numberWithFloat:experienceDelta], [NSNumber numberWithFloat:stressDelta], nil]
                                                               forKeys:[NSArray arrayWithObjects:@"Intelligence", @"Creativity", @"Technology", @"Culture", @"Experience", @"Stress", nil]];
    [self performSelector:@selector(showIntelligenceBoost:) withObject:statBoostDict afterDelay:1.0f];
}

-(void)processTimePassed {
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSReporter * reporter = appDelegate.currentReporter;
    NSDictionary * activity = reporter.currentActivity;
    NSInteger daysPassed = 1;
    if ([activity valueForKey:@"Days"] != nil) {
        //it's travel, there is more than one day
        daysPassed += [[activity valueForKey:@"Days"] intValue];
    }
    [appDelegate setDaysPassed:[appDelegate daysPassed]+daysPassed];
}

-(void)configureLabel:(UILabel *)label withValue:(CGFloat)value {
    if (value > 0.1f) {
        [label setTextColor:[UIColor greenColor]];
        [label setText:[NSString stringWithFormat:@"+%.1f",value]];
    }
    else if(value < -0.1f) {
        [label setTextColor:[UIColor redColor]];
        [label setText:[NSString stringWithFormat:@"%.1f",value]];
    }
    else {
        [label setText:@"No Change"];
        [label setTextColor:[UIColor blueColor]];
    }
    [UIView animateWithDuration:0.5f
                     animations:^(void){
                         [label setAlpha:1.0f];
                     }];
}

-(void)showIntelligenceBoost:(NSDictionary*)statDict {
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSReporter * reporter = appDelegate.currentReporter;
    CGFloat val = [[statDict valueForKey:@"Intelligence"] floatValue];
    UILabel * label = self.intelligenceDeltaLabel;
    [self configureLabel:label withValue:val];
    [self performSelector:@selector(showCreativityBoost:) withObject:statDict afterDelay:1.0f];
    [self.intelligenceBar setProgress:reporter.intelligence/10 animated:YES];
}

-(void)showCreativityBoost:(NSDictionary*)statDict{
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSReporter * reporter = appDelegate.currentReporter;
    CGFloat val = [[statDict valueForKey:@"Creativity"] floatValue];
    UILabel * label = self.creativityDeltaLabel;
    [self configureLabel:label withValue:val];
    [self performSelector:@selector(showTechnologyBoost:) withObject:statDict afterDelay:1.0f];
    [self.creativityBar setProgress:reporter.creativity/10 animated:YES];
}
-(void)showTechnologyBoost:(NSDictionary*)statDict{
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSReporter * reporter = appDelegate.currentReporter;
    CGFloat val = [[statDict valueForKey:@"Technology"] floatValue];
    UILabel * label = self.technologyDeltaLabel;
    [self configureLabel:label withValue:val];
    [self performSelector:@selector(showCultureBoost:) withObject:statDict afterDelay:1.0f];
    [self.technologyBar setProgress:reporter.technology/10 animated:YES];
}
-(void)showCultureBoost:(NSDictionary*)statDict{
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSReporter * reporter = appDelegate.currentReporter;
    CGFloat val = [[statDict valueForKey:@"Culture"] floatValue];
    UILabel * label = self.cultureDeltaLabel;
    [self configureLabel:label withValue:val];
    [self performSelector:@selector(showExperienceBoost:) withObject:statDict afterDelay:1.0f];
    [self.cultureBar setProgress:reporter.culture/100 animated:YES];
}
-(void)showExperienceBoost:(NSDictionary*)statDict {
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSReporter * reporter = appDelegate.currentReporter;
    CGFloat val = [[statDict valueForKey:@"Experience"] floatValue];
    UILabel * label = self.experienceDeltaLabel;
    [self configureLabel:label withValue:val];
    [self performSelector:@selector(showStressBoost:) withObject:statDict afterDelay:1.0f];
    [self.experienceBar setProgress:reporter.experience/100 animated:YES];
}
-(void)showStressBoost:(NSDictionary*)statDict {
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSReporter * reporter = appDelegate.currentReporter;
    CGFloat val = [[statDict valueForKey:@"Stress"] floatValue];
    UILabel * label = self.stressDeltaLabel;
    [self configureLabel:label withValue:val];
    [label setTextColor:(val>0.1f)?[UIColor redColor]:(val < -0.1f)?[UIColor greenColor]:[UIColor blueColor]];
    [self.stressBar setProgress:reporter.stress/100 animated:YES];
    [UIView animateWithDuration:0.5f animations:^(){[self.sleepButton setAlpha:1.0f];}];
    [self makeCharacterHappy];
}

-(void)makeCharacterHappy {
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSReporter * reporter = appDelegate.currentReporter;
    [[self characterView] setAnimationImages:[reporter excitedImages]];
    [[self characterView] startAnimating];
}

#pragma mark - View lifecycle

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self initializeAppearance];
    [self processDailyEvent];
    [self processTimePassed];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    [appDelegate setActivityBonusName:nil];
    [appDelegate setActivityBonusAmount:0];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait || interfaceOrientation == UIInterfaceOrientationPortraitUpsideDown);
}

- (IBAction)goToSleep:(id)sender {
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    UIViewController * calendarView = [[self.navigationController viewControllers] objectAtIndex:1];
    if ([appDelegate daysPassed] >= appDelegate.currentJourney.duration) {
        [self.navigationController popToViewController:calendarView animated:NO];
        [calendarView performSegueWithIdentifier:@"showGameOverScreen" sender:self];
    }
    else {
        [self.navigationController popToViewController:calendarView animated:YES];
    }
}

- (void)viewDidUnload {
    [self setIntelligenceBar:nil];
    [self setCreativityBar:nil];
    [self setTechnologyBar:nil];
    [self setCultureBar:nil];
    [self setExperienceBar:nil];
    [self setStressBar:nil];
    [self setIntelligenceDeltaLabel:nil];
    [self setCreativityDeltaLabel:nil];
    [self setTechnologyDeltaLabel:nil];
    [self setCultureDeltaLabel:nil];
    [self setExperienceDeltaLabel:nil];
    [self setStressDeltaLabel:nil];
    [self setMoneyDeltaLabel:nil];
    [self setCharacterView:nil];
    [self setSleepButton:nil];
    [super viewDidUnload];
}

-(CGFloat)randRatio {
    CGFloat answer = (random()%100)/100;
    return answer;
}

@end
