//
//  TSJourneyView.m
//  TravelSection
//
//  Created by Michael Herring on 12/17/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "TSJourneyView.h"
#import "TSMinigameViewController.h"

enum TSTravelSheetType {
    TSTravelSheetLocations = 50,
    TSTravelSheetMethods
};

@implementation TSJourneyView
@synthesize mapView;

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSContinentalJourney * journey = [appDelegate currentJourney];
    MKCoordinateRegion newRegion = [journey region];
    [[self mapView] setRegion:newRegion];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationLandscapeLeft || interfaceOrientation == UIInterfaceOrientationLandscapeRight);
}

- (void)viewDidUnload {
    [self setMapView:nil];
    [super viewDidUnload];
}

- (IBAction)dismissSelf:(id)sender {
    [self dismissModalViewControllerAnimated:YES];
}
- (IBAction)pickDestination:(id)sender {
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSContinentalJourney * journey = appDelegate.currentJourney;
    TSReporter * reporter = appDelegate.currentReporter;
    NSArray * availableConnections = [journey.cityConnections valueForKey:reporter.currentCity];
    NSString * destination1 = ([availableConnections count] > 0)?[availableConnections objectAtIndex:0]:nil;
    NSString * destination2 = ([availableConnections count] > 1)?[availableConnections objectAtIndex:1]:nil;
    NSString * destination3 = ([availableConnections count] > 2)?[availableConnections objectAtIndex:2]:nil;
    if (destination1 == nil && destination2 == nil && destination3 == nil) {
        UIAlertView * alreadyThereAlert = [[UIAlertView alloc] initWithTitle:@"Already There!" message:@"You've already made it to your destination! Do some jobs and have some cultural experiences so your article will be great!" delegate:nil cancelButtonTitle:@"Okay" otherButtonTitles:nil];
        [alreadyThereAlert show];
    }
    else {
        UIActionSheet * travelSheet = [[UIActionSheet alloc]initWithTitle:@"To Which City?" delegate:self cancelButtonTitle:@"Nevermind" destructiveButtonTitle:nil otherButtonTitles:destination1, destination2, destination3, nil];
        [travelSheet setTag:TSTravelSheetLocations];
        [travelSheet showInView:self.view];
    }
}

-(void)pickTravelMethod:(NSInteger)selectedDestinationIndex {
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSContinentalJourney * journey = appDelegate.currentJourney;
    TSReporter * reporter = appDelegate.currentReporter;
    _selectedDestinationCity = [[journey.cityConnections valueForKey:reporter.currentCity] objectAtIndex:selectedDestinationIndex];
    NSArray * availableMethods = [journey pricesToTravelFrom:reporter.currentCity to:_selectedDestinationCity];
    NSString * method1 = nil, *method2 = nil, *method3 = nil, *formatString = @"%@, %i days, $%.2f";
    NSDictionary * nextMethod = nil;
    if ([availableMethods count] > 0) {
        nextMethod = [availableMethods objectAtIndex:0];
        method1 = [NSString stringWithFormat:formatString,[nextMethod valueForKey:@"TravelMethod"], [[nextMethod valueForKey:@"Days"] intValue], fabs([[nextMethod valueForKey:@"MoneyDelta"] floatValue])];
    }
    if ([availableMethods count] > 1) {
        nextMethod = [availableMethods objectAtIndex:1];
        method2 = [NSString stringWithFormat:formatString,[nextMethod valueForKey:@"TravelMethod"], [[nextMethod valueForKey:@"Days"] intValue], fabs([[nextMethod valueForKey:@"MoneyDelta"] floatValue])];
    }
    if ([availableMethods count] > 2) {
        nextMethod = [availableMethods objectAtIndex:2];
        method3 = [NSString stringWithFormat:formatString,[nextMethod valueForKey:@"TravelMethod"], [[nextMethod valueForKey:@"Days"] intValue], fabs([[nextMethod valueForKey:@"MoneyDelta"] floatValue])];
    }
    UIActionSheet * travelSheet = [[UIActionSheet alloc]initWithTitle:@"How are you getting there?" delegate:self cancelButtonTitle:@"Nevermind" destructiveButtonTitle:nil otherButtonTitles:method1,method2,method3, nil];
    [travelSheet setTag:TSTravelSheetMethods];
    [travelSheet showInView:self.view];
}

-(void)doTheTravel {
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSReporter * reporter = appDelegate.currentReporter;
    NSMutableDictionary * tempDict = [NSMutableDictionary dictionaryWithObject:_selectedDestinationCity forKey:@"Destination"];
    [tempDict addEntriesFromDictionary:_finalDestinationAndMethod];
    [reporter setCurrentActivity:[NSDictionary dictionaryWithDictionary:tempDict]];
    
    //now pushity-pop lol
    UINavigationController * navController = (UINavigationController*)appDelegate.window.rootViewController;
    
    UIViewController * calendarController = [navController.viewControllers objectAtIndex:1];
    [self dismissModalViewControllerAnimated:NO];
    [calendarController performSegueWithIdentifier:@"beginActivity" sender:self];
}

#pragma mark - UIActionSheetDelegate implementation

- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex {
    if ([actionSheet cancelButtonIndex] == buttonIndex) return;
    if ([actionSheet tag] == TSTravelSheetLocations) {
        [self pickTravelMethod:buttonIndex];
    }
    else /* Methods */ {
        TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
        TSContinentalJourney * journey = appDelegate.currentJourney;
        TSReporter * reporter = appDelegate.currentReporter;
        _finalDestinationAndMethod = [[journey pricesToTravelFrom:reporter.currentCity to:_selectedDestinationCity] objectAtIndex:buttonIndex];
        if (fabs([[_finalDestinationAndMethod valueForKey:@"MoneyDelta"] floatValue]) > reporter.money) {
            UIAlertView * moneyAlert = [[UIAlertView alloc] initWithTitle:@"Not Enough Money!" message:@"You need more cash before you can take that trip! Try taking some jobs, or a cheaper method of travel." delegate:nil cancelButtonTitle:@"Okay" otherButtonTitles:nil];
            [moneyAlert show];
            return;
        }
        [self doTheTravel];
    }
}

@end
