//
//  TSMinigameViewController.m
//  TravelSection
//
//  Created by Michael Herring on 12/17/11.
//  Copyright (c) 2011 Sun, Sea and Sky Factory. All rights reserved.
//

#import "TSMinigameViewController.h"
#import "TSDailySummaryViewController.h"

@implementation TSMinigameViewController

@synthesize activityInfoLabel = _activityInfoLabel;
@synthesize finishButton = _finishButton;
@synthesize backgroundImage = _backgroundImage;
@synthesize characterImage = _characterImage;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    TSReporter * reporter = appDelegate.currentReporter;
    TSContinentalJourney * journey = appDelegate.currentJourney;
    [self.characterImage setImage:[reporter stationaryImage]];
    [self.characterImage.layer setMagnificationFilter:kCAFilterNearest];
    NSString * activityName = [reporter.currentActivity valueForKey:@"ActivityName"];
    NSString * resultText = nil;
    if ([reporter.currentActivity valueForKey:@"TravelMethod"] != nil) {
        resultText = [NSString stringWithFormat:@"You decided to travel by %@.",[reporter.currentActivity valueForKey:@"TravelMethod"]];
        [[self activityInfoLabel] setText:resultText];
    }
    else {
        [[self activityInfoLabel] setText:[journey instructionStringForActivity:activityName]];
    }

    
    switch ([journey minigameTypeForActivity:activityName]) {
        case TSMinigameTapIncomingItems:
            [self beginTapIncomingGame];
            break;
        case TSMinigameJumpOverOncomingObjects:
            [self beginJumpOverObjectsGame];
            break;
        case TSMinigameFlickItemsOutward:
            [self beginFlickItemsGame];
            break;
        case TSMinigameDodgeOncomingObjects:
            [self beginDodgeObjectsGame];
            break;
        case TSMinigameJustWatch:
        default:
            [self showFinishButton:2.5f];
            break;
    }
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
    [_timer invalidate];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationLandscapeLeft || interfaceOrientation == UIInterfaceOrientationLandscapeRight);
}


- (void)viewDidUnload {
    [self setActivityInfoLabel:nil];
    [self setFinishButton:nil];
    [self setBackgroundImage:nil];
    [self setCharacterImage:nil];
    [super viewDidUnload];
}

- (IBAction)completeActivity:(id)sender {
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    UINavigationController * navController = (UINavigationController*)appDelegate.window.rootViewController;
    UIViewController * calendarController = [navController.viewControllers objectAtIndex:1];
    [self dismissModalViewControllerAnimated:NO];
    [calendarController performSegueWithIdentifier:@"showSummary" sender:self];
}

#pragma mark - minigame implementations

-(void)beginTapIncomingGame {
    CGRect offscreenRect = CGRectMake(self.view.bounds.size.width, self.view.bounds.size.height/2, 50, 75);
    //what are incoming, and how many?
    unsigned int numberOfIncoming = rand() % 20;
    NSString * incomingName = @"Wine";
    //build the incoming items array
    _activeObjects = [NSMutableArray arrayWithCapacity:numberOfIncoming];
    for (int i=0; i<numberOfIncoming; ++i) {
        TSTappableImageView * nextImageView = [[TSTappableImageView alloc] initWithImage:[UIImage imageNamed:incomingName]];
        [self.view addSubview:nextImageView];
        [nextImageView setFrame:offscreenRect];
        [nextImageView setUserInteractionEnabled:YES];
        [nextImageView setTarget:self];
        [nextImageView setAction:@selector(incomingTapped)];
        [_activeObjects addObject:nextImageView];
    }
    //make 'em fly
    _timer = [NSTimer scheduledTimerWithTimeInterval:(NSTimeInterval)((1 / 60.0) )
                                              target:self 
                                            selector:@selector(incomingUpdate:)
                                            userInfo:nil
                                             repeats:TRUE];
    //configure the bonuses
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    [appDelegate setActivityBonusAmount:0];
    [appDelegate setActivityBonusName:@"Culture"];
}

static CGFloat incomingSpeed = 5.0f;

-(void)incomingUpdate:(id)sender {
    if ([_activeObjects count] <= 0) {
        [self showFinishButton:0.0f];
        return;
    }
    UIImageView * imageView = [_activeObjects objectAtIndex:0];
    [imageView setCenter:CGPointMake(imageView.center.x-incomingSpeed, imageView.center.y)];
    if (imageView.center.x < -25.0f) {
        [imageView removeFromSuperview];
        [_activeObjects removeObjectAtIndex:0];
        incomingSpeed = 5.0f + (float)(rand()%100)/10.0f;
    }
}

-(void)incomingTapped {
    TSAppDelegate * appDelegate = [[UIApplication sharedApplication] delegate];
    appDelegate.activityBonusAmount++;
    [_activeObjects removeObjectAtIndex:0];
    incomingSpeed = 5.0f + (float)(rand()%100)/10.0f;
}

-(void)beginJumpOverObjectsGame {
    
}

-(void)beginFlickItemsGame {
    
}

-(void)beginDodgeObjectsGame {
    
}

-(void)showFinishButton:(NSTimeInterval)withDelay {
    [UIView animateWithDuration:1.25f delay:withDelay options:UIViewAnimationOptionAllowUserInteraction animations:^(){[self.finishButton setAlpha:1.0f];} completion:^(BOOL finished){}];
}

@end
